import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;



public class TSP {
	public static void main(String[] args) {
		long prgStart = System.currentTimeMillis();
		long prgEnd = prgStart;
		String appName = "tsp";
		
		// Handle command-line switches
		Options options = new Options();
		Option seedOpt = OptionBuilder.withArgName("seed")
					.hasOptionalArg()
					.withDescription("Use this seed for the random number generator.")
					.create("s");
		Option inputOpt = OptionBuilder.withArgName("input file")
						.hasArg()
						.withDescription("Read this file as a problem instance.")
						.create("i");
		Option outputOpt = OptionBuilder.withArgName("output file")
						.hasOptionalArg()
						.withDescription("Print the solution to this file.")
						.create("o");
		Option timeOpt = OptionBuilder.withArgName("time limit")
						.hasOptionalArg()
						.withDescription("Time limit for the program in seconds.")
						.create("t");
		Option chOpt = OptionBuilder.withArgName("construction algorithm")
					.hasOptionalArg()
					.withDescription("The algorithm to use for creating the initial solution.")
					.create("ch");
		Option verboseOpt = OptionBuilder.withDescription("Print extra information.")
						.create("v");
		Option helpOpt = new Option("h", "Print this message.");
		
		options.addOption(seedOpt);
		options.addOption(inputOpt);
		options.addOption(outputOpt);
		options.addOption(timeOpt);
		options.addOption(chOpt);
		options.addOption(verboseOpt);
		options.addOption(helpOpt);
		
		HelpFormatter help = new HelpFormatter();
		
		CommandLineParser parser = new PosixParser();
		CommandLine cmd = null;
		try {
			cmd = parser.parse(options, args);
		} catch (ParseException e) {
			help.printHelp(appName, options);
			System.exit(1);
		}
		
		if (cmd.hasOption("h")) {
			help.printHelp(appName, options);
			System.exit(0);
		}
		
		if (!cmd.hasOption("i")) {
			System.err.println("An input file must be supplied.");
			System.exit(1);
		}
		Problem p = new Problem();
		try {
			p.loadFromFile(cmd.getOptionValue("i"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
			
		if (cmd.hasOption("t")) {
			prgEnd += Integer.parseInt(cmd.getOptionValue("t")) * 1000;
			System.out.println("TIMELIMIT: "
					+ Integer.parseInt(cmd.getOptionValue("t")));
		} else {
			prgEnd += 5000;
			System.out.println("TIMELIMIT: 5");
		}
		
			
		long seed = System.currentTimeMillis();
		if (cmd.hasOption("s")) {
			String seedStr = cmd.getOptionValue("s");
			try {
				seed = Long.parseLong(seedStr);
			} catch (NumberFormatException e) {
				System.err.println("The seed must be parsable by Long.parseLong.");
				System.exit(1);
			}
		}
		
		boolean verbose = false;
		if (cmd.hasOption("v")) {
			verbose = true;
		}
		
		Random r = new Random(seed);
		TourSearch ch = new TourSearch();
		Tour tour = new Tour(p);
		
		if (cmd.hasOption("ch")) {
			String chOptString = cmd.getOptionValue("ch");
			if (chOptString.equals("Random")) {
				ch = new TourRandom(p,verbose);
			} else if (chOptString.equals("YourAlg")) {
			    ch = new TourYourAlg(p,verbose);
			    //TODO Implement in TourYourAlg your Heuristic
			    // look at TourRandom and imitate the structure
			} else {
				System.out.println("Option 'ch' not valid (" + chOptString + ").");
				System.exit(1);
				// to make the compiler happy
				ch = new TourRandom(p,verbose);
			}
		} else {
			ch = new TourRandom(p,verbose);
		}
		
		// Print info of the run
		System.out.println("INSTANCE: " + cmd.getOptionValue("i"));
		System.out.println("SEED: " + seed);
		if (cmd.hasOption("o")) {
			System.out.println("OUTPUTFILE: " + cmd.getOptionValue("o"));
		}
		System.out.println("HEURISTIC: " + cmd.getOptionValue("ch"));
		
		// Solve
		long start, end;
		
		// START
		start = System.currentTimeMillis();
		ch.construct(tour, r);
		tour.computeLength();
		end = System.currentTimeMillis();
		System.out.println("TOURLENGTH: " + tour.getLength());
		System.out.println("TIME: " + (double) (end - start)/1000);
		// END
		
		// Print solution
		if (cmd.hasOption("o")) {
			try {
			    String sol_file = cmd.getOptionValue("o");
				PrintWriter pw = new PrintWriter(new File(sol_file));
				pw.print(tour);
				pw.close();
			} catch (FileNotFoundException e) {
				System.err.println("Output file not found.");
				System.exit(1);
			}
		} else {
			// System.out.println(s);
		}
		
	}
}


